@@ -1,8 +1,9 @@
-# Copyright 1999-2014 Gentoo Foundation
+# Copyright 1999-2018 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 from __future__ import unicode_literals
 
+import functools
 import io
 
 import _emerge.emergelog
@@ -23,6 +24,8 @@ from portage import _encodings, _unicode_decode, _unicode_encode, os
 from portage.package.ebuild.digestcheck import digestcheck
 from portage.package.ebuild.doebuild import _check_temp_dir
 from portage.package.ebuild._spawn_nofetch import SpawnNofetchWithoutBuilddir
+from portage.util._async.AsyncTaskFuture import AsyncTaskFuture
+
 
 class EbuildBuild(CompositeTask):
 
@@ -32,23 +35,33 @@ class EbuildBuild(CompositeTask):
 		("_build_dir", "_buildpkg", "_ebuild_path", "_issyspkg", "_tree")
 
 	def _start(self):
-
-		pkg = self.pkg
-		settings = self.settings
-
 		if not self.opts.fetchonly:
-			rval = _check_temp_dir(settings)
+			rval = _check_temp_dir(self.settings)
 			if rval != os.EX_OK:
 				self.returncode = rval
 				self._current_task = None
 				self._async_wait()
 				return
 
+		# First get the SRC_URI metadata (it's not cached in self.pkg.metadata
+		# because some packages have an extremely large SRC_URI value).
+		self._start_task(
+			AsyncTaskFuture(
+				future=self.pkg.root_config.trees["porttree"].dbapi.\
+				async_aux_get(self.pkg.cpv, ["SRC_URI"], myrepo=self.pkg.repo,
+				loop=self.scheduler)),
+			self._start_with_metadata)
+
+	def _start_with_metadata(self, aux_get_task):
+		self._assert_current(aux_get_task)
+		pkg = self.pkg
+		settings = self.settings
 		root_config = pkg.root_config
 		tree = "porttree"
 		self._tree = tree
 		portdb = root_config.trees[tree].dbapi
 		settings.setcpv(pkg)
+		settings.configdict["pkg"]["SRC_URI"], = aux_get_task.future.result()
 		settings.configdict["pkg"]["EMERGE_FROM"] = "ebuild"
 		if self.opts.buildpkgonly:
 			settings.configdict["pkg"]["MERGE_TYPE"] = "buildonly"
@@ -147,8 +160,13 @@ class EbuildBuild(CompositeTask):
 
 		self._build_dir = EbuildBuildDir(
 			scheduler=self.scheduler, settings=settings)
-		self._build_dir.lock()
+		self._start_task(
+			AsyncTaskFuture(future=self._build_dir.async_lock()),
+			self._start_pre_clean)
 
+	def _start_pre_clean(self, lock_task):
+		self._assert_current(lock_task)
+		lock_task.future.result()
 		# Cleaning needs to happen before fetch, since the build dir
 		# is used for log handling.
 		msg = " === (%s of %s) Cleaning (%s::%s)" % \
@@ -185,8 +203,7 @@ class EbuildBuild(CompositeTask):
 
 	def _pre_clean_exit(self, pre_clean_phase):
 		if self._default_exit(pre_clean_phase) != os.EX_OK:
-			self._unlock_builddir()
-			self.wait()
+			self._async_unlock_builddir(returncode=self.returncode)
 			return
 
 		# for log handling
@@ -200,8 +217,14 @@ class EbuildBuild(CompositeTask):
 			logfile=self.settings.get('PORTAGE_LOG_FILE'),
 			pkg=self.pkg, scheduler=self.scheduler)
 
+		self._start_task(AsyncTaskFuture(
+			future=fetcher.async_already_fetched(self.settings)),
+			functools.partial(self._start_fetch, fetcher))
+
+	def _start_fetch(self, fetcher, already_fetched_task):
+		self._assert_current(already_fetched_task)
 		try:
-			already_fetched = fetcher.already_fetched(self.settings)
+			already_fetched = already_fetched_task.future.result()
 		except portage.exception.InvalidDependString as e:
 			msg_lines = []
 			msg = "Fetch failed for '%s' due to invalid SRC_URI: %s" % \
@@ -209,10 +232,7 @@ class EbuildBuild(CompositeTask):
 			msg_lines.append(msg)
 			fetcher._eerror(msg_lines)
 			portage.elog.elog_process(self.pkg.cpv, self.settings)
-			self.returncode = 1
-			self._current_task = None
-			self._unlock_builddir()
-			self.wait()
+			self._async_unlock_builddir(returncode=1)
 			return
 
 		if already_fetched:
@@ -283,8 +303,7 @@ class EbuildBuild(CompositeTask):
 
 		if 'fetch' not in self.pkg.restrict and \
 			'nofetch' not in self.pkg.defined_phases:
-			self._unlock_builddir()
-			self.wait()
+			self._async_unlock_builddir(returncode=self.returncode)
 			return
 
 		self.returncode = None
@@ -294,18 +313,32 @@ class EbuildBuild(CompositeTask):
 
 	def _nofetch_exit(self, nofetch_phase):
 		self._final_exit(nofetch_phase)
-		self._unlock_builddir()
-		self.returncode = 1
-		self.wait()
+		self._async_unlock_builddir(returncode=1)
 
-	def _unlock_builddir(self):
+	def _async_unlock_builddir(self, returncode=None):
+		"""
+		Release the lock asynchronously, and if a returncode parameter
+		is given then set self.returncode and notify exit listeners.
+		"""
+		if returncode is not None:
+			# The returncode will be set after unlock is complete.
+			self.returncode = None
 		portage.elog.elog_process(self.pkg.cpv, self.settings)
-		self._build_dir.unlock()
+		self._start_task(
+			AsyncTaskFuture(future=self._build_dir.async_unlock()),
+			functools.partial(self._unlock_builddir_exit, returncode=returncode))
+
+	def _unlock_builddir_exit(self, unlock_task, returncode=None):
+		self._assert_current(unlock_task)
+		# Normally, async_unlock should not raise an exception here.
+		unlock_task.future.result()
+		if returncode is not None:
+			self.returncode = returncode
+			self._async_wait()
 
 	def _build_exit(self, build):
 		if self._default_exit(build) != os.EX_OK:
-			self._unlock_builddir()
-			self.wait()
+			self._async_unlock_builddir(returncode=self.returncode)
 			return
 
 		buildpkg = self._buildpkg
@@ -370,8 +403,7 @@ class EbuildBuild(CompositeTask):
 		"""
 
 		if self._default_exit(packager) != os.EX_OK:
-			self._unlock_builddir()
-			self.wait()
+			self._async_unlock_builddir(returncode=self.returncode)
 			return
 
 		if self.opts.buildpkgonly:
@@ -425,8 +457,9 @@ class EbuildBuild(CompositeTask):
 	def _clean_exit(self, clean_phase):
 		if self._final_exit(clean_phase) != os.EX_OK or \
 			self.opts.buildpkgonly:
-			self._unlock_builddir()
-		self.wait()
+			self._async_unlock_builddir(returncode=self.returncode)
+		else:
+			self.wait()
 
 	def create_install_task(self):
 		"""
@@ -461,4 +494,14 @@ class EbuildBuild(CompositeTask):
 		return task
 
 	def _install_exit(self, task):
-		self._unlock_builddir()
+		"""
+		@returns: Future, result is the returncode from an
+			EbuildBuildDir.async_unlock() task
+		"""
+		self._async_unlock_builddir()
+		if self._current_task is None:
+			result = self.scheduler.create_future()
+			self.scheduler.call_soon(result.set_result, os.EX_OK)
+		else:
+			result = self._current_task.async_wait()
+		return result
